package decatur.de.rigorous.ivp;

import java.applet.Applet; 
import java.awt.*;
import decatur.de.rigorous.numutil.*;
import decatur.de.rigorous.plot2d.BoxedPanel;
import decatur.de.rigorous.plot2d.Plot2d;
import decatur.de.rigorous.parse.*;

class SmallPlots extends BoxedPanel {
    Ivp applet;
    Choice choice;
    Checkbox optimized;
    
    
    SmallPlots (Ivp applet) {
	super ();
	this.applet = applet;
	
	choice = new Choice();
	for (int i=0; i<applet.models.length; i++)
	    choice.addItem(applet.models[i][0]);
	choice.select(0);
	
	optimized = new Checkbox ("optimized");
	optimized.setState (true);
	
	setFill (GridBagConstraints.NONE);
	add (1,1,0,0,choice);
	add (1,1,0,1,optimized);
	
    }
    
    boolean getMethod () {
	return optimized.getState ();
    }

    public boolean handleEvent(Event e) {
	if (e.id == Event.ACTION_EVENT) {
	    if (e.target==choice) {
		applet.selectModel(choice.getSelectedIndex());
		return true;
	    } else if (e.target instanceof Checkbox) {
		applet.z.setOptimized (getMethod());
		return true;
	    }
	}
	return false;
    } 
}

class MathControl extends BoxedPanel {
    TextField func[],
	xmin, xmax, ymin, ymax,
	xcenter, ycenter, radius, numz;
    Button stopbutton, stepbutton, erase, set1, set2;
    Iter iter = null;
    TextField steps, counter, skip;
    TextArea message;
    Ivp applet;
    
    MathControl (Ivp applet) {
	super ();
	this.applet = applet;
	func = new TextField[2];
	func[0] = new TextField("", 20);
	func[1] = new TextField("", 20);
	xmin  = new TextField("", 8);
	xmax = new TextField("", 8);
	ymin  = new TextField("", 8);
	ymax  = new TextField("", 8);
	xcenter  = new TextField("", 8);
	ycenter  = new TextField("", 8);
	radius =   new TextField("", 8);
	numz =     new TextField("", 1);
	steps = new TextField ("", 3);
	skip = new TextField ("", 3);
	counter = new TextField ("", 3);
	counter.setEditable (false);
	stepbutton  = new Button ("do steps");
	stopbutton  = new Button ("stop");
	erase = new Button("erase");
	set1 = new Button ("set");
	set2 = new Button ("set");
	message = new TextArea("", 3, 20);
	
	BoxedPanel p1 = new BoxedPanel();
	
	p1.add (1,1,0,0, new Label("min x =", Label.RIGHT));
	p1.add (2,1,1,0, xmin);
	
	p1.add (1,1,3,0, new Label("max x =", Label.RIGHT));
	p1.add (2,1,4,0, xmax);
	
	
	p1.add (1,1,0,1, new Label("min y =", Label.RIGHT));
	p1.add (2,1,1,1, ymin);
	
	p1.add (1,1,3,1, new Label("max y =", Label.RIGHT));
	p1.add (2,1,4,1, ymax);
	
	p1.add (1,1,0,2, new Label("x->", Label.RIGHT));
	p1.add (3,1,1,2, func[0]);
	p1.add (1,1,0,3, new Label("y->", Label.RIGHT));
	p1.add (3,1,1,3, func[1]);
	p1.add (1,1,4,2, new Label("order=", Label.RIGHT));
	p1.add (1,1,5,2, numz);
	p1.add (2,1,4,3, set1);
	
	BoxedPanel p2 = new BoxedPanel();
	
	p2.add (1,1,0,0, new Label("x0=", Label.RIGHT));
	p2.add (2,1,1,0, xcenter);
	p2.add (1,1,0,1, new Label("y0=", Label.RIGHT));
	p2.add (2,1,1,1, ycenter);
	p2.add (1,1,0,2, new Label("r0=", Label.RIGHT));
	p2.add (2,1,1,2, radius);
	p2.add (3,1,0,3, set2);
	
	BoxedPanel p3 = new BoxedPanel();
	
	p3.add (1,1,0,0, new Label("count=", Label.RIGHT));
	p3.add (1,1,1,0, counter);
	p3.add (1,1,0,1, new Label("steps=", Label.RIGHT));
	p3.add (1,1,1,1, steps);
	p3.add (1,1,2,0, stepbutton); 
	p3.add (1,1,2,1, stopbutton); 
	p3.add (1,1,3,0, erase);
	//	p3.add (1,1,3,1, skip);
	p3.add (7,2,4,0, message); 
	
	add(4,4,0,0,p1);
	add(2,4,4,0,p2);
	add(6,2,0,4,p3);
	
	show ();
    }
    
    public Dimension preferredSize() {
	return minimumSize();
    }
    
    public synchronized Dimension minimumSize() {
	return new Dimension (applet.size().width, applet.size().height-400-10);
    }
    
    public void Bounds (String infx, String supx, String infy, String supy) {
	xmin.setText(infx);
	xmax.setText(supx);
	ymin.setText(infy);
	ymax.setText(supy);
    }
    
    public boolean handleEvent(Event e) {
	if (e.id == Event.ACTION_EVENT) {
	    if (e.target==stopbutton) {
		if ( iter!=null ) iter.suspendSoon = true;
		return true;
	    } else if (e.target==stepbutton) {
		if ( iter==null || !iter.isAlive () ) {
		    iter = new Iter(this, applet);
		    iter.start ();
		}
		return true;
	    } else if (e.target==erase) {
		applet.plot1.erase ();
		applet.plot1.drawAxes ();
		applet.plot1.repaint ();
		return true;
	    } else if (e.target==set1) {
		applet.reset1 ();
		applet.reset2 ();
		return true;
	    } else if (e.target==set2) {
		applet.reset2 ();
		return true;
	    }
	}
	return false;
    }  

    public void paint(Graphics g) {
    }
}

class Iter extends Thread {
    MathControl panel;
    Ivp ivp;
    private  int stage, stages, count;
    boolean suspendSoon;

    Iter (MathControl panel, Ivp ivp) {
	super ();
	this.panel = panel;
	this.ivp = ivp;
    }
    
    public void start () {
	stages = Integer.parseInt (panel.steps.getText());
	stage =  Integer.parseInt (panel.counter.getText());
	count = 0;
	suspendSoon = false;
	setPriority (getPriority()-1);
	super.start ();
    }
    
    public void run() {
	while (true) {
	    if ( !ivp.step(count==stages) ) stop ();
	    panel.counter.setText(""+(++stage));
	    if ( ++count==stages || suspendSoon ) stop ();
	    else yield ();
	}
	
    }
}

class MyPlot extends Plot2d {
    MathControl cp;
  
    MyPlot (int devwidth, int devheight) {
	super (devwidth, devheight);
    }
  
    public void drawZonotope (double[][] line, double[] center) {
	int n = line.length;
	save ();
	if (center!=null)
	    translate (center[0], center[1]);

	for (int i=0; i<n; i++) 
	    translate (line[i][0], line[i][1]);

	for (int j=-2; j<=2; j+=4)
	    for (int i=0; i<n; i++) {
		drawLine(0, 0, j*line[i][0], j*line[i][1]);
		translate(j*line[i][0], j*line[i][1]);
	    }
	restore ();
    }
}

public class Ivp extends Applet {
    MyPlot plot1, plot2;
    Plot2d plot3;
    Expression expr[];
    String funcdef[];
    int dim = 2, m;
    int debug = 0;
    double radius[], center[];
    double[][] C, R;
    double radii[];
    int skip;
    Zonotope z, zz;
    MathControl cp;
    String[][] models = 
    {{"Rotation 10","0.9848*x-0.1736*y","0.9848*y+0.1736*x",
      "1", "0", "0.1", "-4", "4","-4","4","5","10"},
     {"Nonlinear 10","0.9848*x-0.1736*y+x^2","0.9848*y+0.1736*x",
      "0", "0", "1e-4", "-1e-3", "1e-3","-1e-3","1e-3","5","10"},
     {"Henon 1","1-2.4*x^2+y","-x",
      "0.4","-0.4","1e-6","0","1","-1","0","5","100"},
     {"Henon 2","1-1.4*x^2+y","0.3*x",
      "0.4","0.4","1e-15","-2","2","-1","1","5","10"},
     {"Chebyshev","y","2*0.8*y-x","1","0.8","0","-2","2","-2","2","5","10"},
     {"Cremona","x*cos(1.3)-(y-x^2)*sin(1.3)","x*sin(1.3)+(y-x^2)*cos(1.3)",
      "-0.38","0.0","0","-1","1","-1","1","5","50"},
     {"Logistic","y","2.005*y*(1-x)","0.7","0.5","0","0","1","0","1","5","100"},
     {"Rotation 45",
      "0.707106*x-0.707106*y+[0,0.001]",
      "0.707106*y+0.707106*x",
      "0.0","0.0","0.0",
      "-4","4","-4","4","3","100"}};
   
    SmallPlots pp;


    public void init () {
	expr = new Expression[2];
	funcdef = new String[2];
	funcdef[0] = "";
	funcdef[1] = "";
	C = new double[dim][dim];
	R = new double[dim][dim];
	radii = new double[m];
	center = new double[dim];
	radius = new double[dim];
	plot1  = new MyPlot (400, 400);
	plot2  = new MyPlot (150, 150);
	plot2.setXmode (Plot2d.NONE); 
	plot2.setYmode (Plot2d.NONE); 
	plot3  = new Plot2d (150, 150);
	cp = new MathControl (this);
	add (plot1);
	pp = new SmallPlots (this);
	pp.add (1,1,0,3,plot2);
	pp.add (1,1,0,4,plot3);
	add (pp);
	add (cp);
	selectModel (0);
	validate ();
    }
    
    void selectModel (int i) {
	String s[] = models[i];
	cp.func[0].setText(s[1]);
	cp.func[1].setText(s[2]);
	cp.Bounds (s[6], s[7], s[8], s[9]);
	cp.xcenter.setText(s[3]);
	cp.ycenter.setText(s[4]);
	cp.radius.setText(s[5]);
	cp.numz.setText(s[10]);
	cp.steps.setText(s[11]);
	cp.skip.setText ("1");
	reset1 ();
	plot1.erase ();
	plot1.drawAxes ();
	reset2 ();
    }
 
    public boolean handleEvent(Event e) {
	
	if ( e.id == Plot2d.POINT_SELECT ) {
	    // evt.arg.getClass().getName().equals("[Ljava.lang.String;") is true
	    String[] s = (String[]) e.arg;
	    cp.xcenter.setText(s[0]);
	    cp.ycenter.setText(s[1]);
	    reset2 ();
	    return true;
	} else if (  e.id == Plot2d.BOX_SELECT ) {
	    // evt.arg.getClass().getName().equals("[Ljava.lang.String;") is true
	    String[] s = (String[]) e.arg;
	    cp.Bounds (s[0], s[1], s[2], s[3]);
	    return true;
	}
	return super.handleEvent (e);
    }

    boolean step (boolean drawAll) {
	z.ihull();
	Interval[] a = Interval.vector(2); 
	blas.center2Interval (a, center, z.ihull());
	CenteredInterval ci;
	try {
	    for (int i=0; i<dim; i++) {
		expr[i].eval(a);
		for (int j=0; j<dim; j++) {
		    ci = expr[i].partial(j).centered();
		    C[i][j] = ci.center; R[i][j] = ci.radius;
		}
	    }
	    a[0].set(center[0]); a[1].set(center[1]);
	    for (int i=0; i<dim; i++) {
		expr[i].eval(a);
		ci = expr[i].value().centered();
		center[i] = ci.center; radius[i] = ci.radius;
	    }
	} catch ( ArithmeticException e ) {
	    cp.message.appendText("\n"+e.toString());
	    cp.stepbutton.disable();
	    return false;
	}

	if (drawAll) {
	    zz.copy (z);
	    zz.eval (C, R, radius);
	}
	z.eval (C, R, radius);
	if ( !drawZonotopes (true, drawAll) ) return false;
	return true;
    }

    boolean drawZonotopes (boolean do1, boolean do2) {
	double rad = z.preparePlot ();
	if ( rad>10*Math.max(plot1.supx-plot1.infx, plot1.supy-plot1.infy) )
	    return false;

	if (do1) {
	    plot2.setColor(Color.black);
	    if ( rad<Math.min(plot1.res[0], plot1.res[1]) ) {
		plot1.drawPoint (center[0], center[1]);
	    } else
		plot1.drawZonotope(z.getLines(), center);
	
	    plot1.repaint();
	}

	if (do2) {
	    plot2.erase();
	    if ( rad>0.0 ) {
		rad *= 1.05;
		plot2.Bounds (-rad, rad, -rad, rad);
		plot2.setColor(Color.red);
		plot2.drawZonotope(z.getLines(), null);
		zz.preparePlot ();
		plot2.drawZonotope(zz.getLines(), null);
		plot2.drawAxes ();
	    }
	    plot2.repaint ();
      
	    plot3.erase();
      
	    double radii[] = new double[m];
	    z.getRadii (radii);
	    double minrad = 1e-16; // Math.log(z.getAverage());
	    double maxrad = 1e-15;
	    for (int i=0; i<m; i++)
		maxrad = Math.max(maxrad, radii[i]);
	    double log10 = Math.log(10.0);
	    minrad = Math.log(minrad)/log10;
	    maxrad = Math.log(maxrad)/log10;
	    plot3.Bounds (-0.5, m-0.5, minrad, maxrad);

	    plot3.setColor(Color.blue);
	    for (int i=0; i<m; i++)
		if ( radii[i]>=1e-16 )
		    plot3.drawRect(i-0.3, minrad, 0.6,
				   Math.log(radii[i])/log10-minrad,
				   true);
	    plot3.drawAxes ();
	    plot3.repaint();
	}
	return true;
    }

    void reset2 () {
	center[0] = Double.valueOf(cp.xcenter.getText()).doubleValue();
	center[1] = Double.valueOf(cp.ycenter.getText()).doubleValue();
	radius[0] =
	    radius[1] = Double.valueOf(cp.radius.getText()).doubleValue();
	cp.counter.setText("0");
	z = new Zonotope(dim, m);
	zz = new Zonotope(dim, m+1);
	z.setOptimized (pp.getMethod());
	z.debug (debug);
	z.init (radius);
	zz.copy (z);
	drawZonotopes (true, true);
    }

    void reset1 () {
	cp.message.setText("");
	double inf_x = Double.valueOf(cp.xmin.getText()).doubleValue();
	double sup_x = Double.valueOf(cp.xmax.getText()).doubleValue();
	double inf_y = Double.valueOf(cp.ymin.getText()).doubleValue();
	double sup_y = Double.valueOf(cp.ymax.getText()).doubleValue();
	plot1.Bounds (inf_x, sup_x, inf_y, sup_y);
	m = Integer.parseInt(cp.numz.getText());
	if ( m<10 ) {
	    plot3.setXmode (Plot2d.MANUAL);
	    for (int i=0; i<m; i++)
		plot3.addXtick (i, String.valueOf(i+1));
	} else
	    plot3.setXmode (Plot2d.NONE);
	
	cp.stepbutton.enable();
	parsefunc ();
    }
    
    void parsefunc () {
	if ( expr[0]!=null && expr[1]!=null &&
	     funcdef[0].equals(cp.func[0].getText()) &&
	     funcdef[1].equals(cp.func[1].getText()) ) return;

	for (int k=0; k<2; k++)
	    funcdef[k] = new String(cp.func[k].getText());

	String s[] = {"x", "y"};
	try {
	    for (int k=0; k<2; k++)
		expr[k] = new Expression(funcdef[k], s);
	    cp.message.appendText("Parsing successful.\n");
	}
	catch (Exception e) {
	    cp.message.appendText("Reading function:\n"+e.toString());
	    expr[0] = expr[1] = null;
	    cp.stepbutton.disable();
	    return;
	}
    }
}
