package decatur.de.rigorous.numutil;

public class SplineInterpolant {

    public double x[];
    public double y[];
    public double y2[];
    public double u[];
    double ypmin = 0.0, ypmax = 0.0;
    
    public SplineInterpolant (int size) {
	x = new double[size];
	y = new double[size];
	y2 = new double[size];
	u = new double[size];
    }

    public SplineInterpolant (double x[], double y[]) {
	this.x = x;
	this.y = y;
	y2 = new double[x.length];
	u = new double[x.length];
	validate ();
    }
  
    public void setBoundarySlope (double a, double b) {
	ypmin = a; ypmax = b;
    }
    
    public void interpolate (double xi[], double yi[]) {
	
	for (int i=1, k=0; i<x.length; i++) {
	    while ( xi[k]<=x[i] ) {
		yi[k] = splint (i, xi[k]);
		k++;
		if ( k==xi.length ) return;
	    }
	}
    }

    int index = 1;
    public double valueAt (double d) {
	index = ArrayTools.closestIndex (x, d, index);
	return splint (index, d);
    }

    private double splint (int i, double d) {
	double h = x[i]-x[i-1];
	double a = (x[i]-d)/h;
	double b = (d-x[i-1])/h;

	return a*y[i-1]+b*y[i]+(a*(a*a-1.0)*y2[i-1]+b*(b*b-1.0)*y2[i])*h*h/6.0;
    }
	    
    public void validate () {
	int N = x.length;
	y2[0] = -0.5;
	u[0]=3.0/(x[1]-x[0])*(y[1]-y[0])/(x[1]-x[0]-ypmin);

	for (int i=1; i<N-1; i++) {
	    double sig = (x[i]-x[i-1])/(x[i+1]-x[i-1]);
	    double p = sig*y2[i-1]+2.0;
	    y2[i] = (sig-1.0)/p;
	    u[i] = (6.0*((y[i+1]-y[i])/(x[i+1]-x[i])-(y[i]-y[i-1])/
		(x[i]-x[i-1]))/(x[i+1]-x[i-1])-sig*u[i-1])/p;
	}

	double qmax = 0.5;
	u[N-1] = 3.0/(x[N-1]-x[N-2])*(ypmax-(y[N-1]-y[N-2])/(x[N-1]-x[N-2]));
	y2[N-1] = (u[N-1]-qmax*u[N-2])/(qmax*y2[N-2]+1.0);
	
	for (int k=N-2; k>=0; k--)
	    y2[k] = y2[k]*y2[k+1]+u[k];
    }
}
