package decatur.de.rigorous.plot;

import java.io.*;

public class PostScriptDevice extends Device {
    private boolean filled;
    private boolean outlined;
  
    private PrintStream stream;
    private int fontSize = 10;
    private String fontName = "Helvetica";
    private SbVec2f p, q;

    public PostScriptDevice (PrintStream stream, String name,
			     int width, int height) {
	super ();
	this.stream = stream;
    
	issue ("%!PS-Adobe-2.0 EPSF-3.0");
	issue ("%%Title: "+name);
	issue ("%%Creator: plot.PSdevice");
	issue ("%%CreationDate: "+new java.util.Date());
	issue ("%%BoundingBox: 0 0 "+width+" "+height);
	issue ("%%DocumentNeededResources: font "+ fontName);
 	issue ("%%EndComments");
	issue ("/pjdict 20 dict def pjdict begin");
	issue ("/COLOR true def");
	issue ("/m {moveto} bind def");
	issue ("/rm {rmoveto} bind def");
	issue ("/l {lineto} bind def");
	issue ("/rl {rlineto} bind def");
	issue ("/n {newpath} bind def");
	issue ("/c {closepath} bind def");
	issue ("/s {stroke} bind def");
	issue ("/f {fill} bind def");
	issue ("/UserColor {COLOR {setrgbcolor} {pop pop pop} ifelse} def");
	issue ("save");
	setFont (fontName, fontSize);

	p = new SbVec2f ();
	q = new SbVec2f ();
    }
   
    public void close () {
	issue ("restore\nshowpage\n%%EOF");
	stream.close ();
    }
    
    private void issue (String s) {
	stream.println (s);
    }
    
    public void setColor (int color) {
	float r = ((color>>16)&0xFF)/255.0f;
	float g = ((color>>8)&0xFF)/255.0f;
	float b = (color&0xFF)/255.0f;
	issue (""+r+" "+g+" "+b+" UserColor");
    }

    public void setDash (int pattern[]) {
	issue ("[");
	if ( pattern!=null )
	    for (int i=0; i<pattern.length; i++)
		issue (" "+pattern[i]);
	issue ("] 0 setdash");
    }

    void setFilled (boolean flag) {
	filled = flag;
    }
    
    void setOutlined (boolean flag) {
	outlined = flag;
    }
  
    void setFont (String fontName, int fontSize) {
	this.fontName = fontName;
	this.fontSize = fontSize;
	issue ("/"+fontName+" findfont "+fontSize+" scalefont setfont");
    }

    public int stringWidth (String s) {
	return (int) (0.5*fontSize*s.length());
    }
    
    public int getAscent () {
	return (int) (0.8*fontSize);
    }
    
    public void drawString (String s, float x, float y, int align) {
	matrix.transform (p, x, y);
	
	issue (""+p.x+" "+p.y+" moveto ("+s+")");
	
	if ( align==CENTER )
	    issue ("dup stringwidth pop 2.0 div neg 0 rm");
	else if ( align==LEFT )
	    issue ("dup stringwidth pop neg 0 rm");
	
	issue ("show");
    }
    
    void drawLine (float x1, float y1, float x2, float y2) {
	matrix.transform (p, x1, y1);
	matrix.transform (q, x2, y2);
	issue ("n "+p.x+" "+p.y+" m "+q.x+" "+q.y+" l c s");
    }
    
    public void drawPolyLine (float x[], float y[], int start, int length) {
	for (int i=start; i<start+length-1; i++)
	    drawLine (x[i], y[i], x[i+1], y[i+1]);  
    }

    public void drawPolygon (float x[], float y[]) {
	
	matrix.transform (p, x[0], y[0]);
	issue ("n "+p.x+" "+p.y+" m");
	for (int i=1; i<x.length; i++) {
	    matrix.transform (p, x[i], y[i]);
	    issue (""+p.x+" "+p.y+" l");
	}

	if ( outlined ) issue ("c save s restore");
	
	if ( filled ) issue ("f");
	else issue ("s");
    }

    public void drawRect (float x, float y, float dx, float dy) {
	matrix.transform (p, x,y);
	matrix.dtransform (q, dx, dy);

	issue ("n "+p.x+" "+p.y+" m");
	issue (""+q.x+" "+0+" rl");
	issue (""+0+" "+q.y+" rl");
	issue (""+q.x+" neg "+0+" rl c");

	if ( filled )
	    issue ("f");
	else
	    issue ("s");
    }

    public void clipRect (float x, float y, float dx, float dy) {
	matrix.transform (p, x,y);
	matrix.dtransform (q, dx, dy);
	
	issue ("n "+p.x+" "+p.y+" m");
	issue (""+q.x+" "+0+" rl");
	issue (""+0+" "+q.y+" rl");
	issue (""+q.x+" neg "+0+" rl c clip n");
    }
}
