package decatur.de.rigorous.plot2d;

import java.awt.*;
import java.io.*;

import decatur.de.rigorous.numutil.Power10;

class UtilFrame extends Frame {
	PlotDevice device;

	UtilFrame(String title) {
		super(title);
		add(new Button("Stop Printing"));
		pack();
	}

	void setDevice(PlotDevice dev) {
		this.device = dev;
	}

	public boolean action(Event e, Object arg) {
		if (arg.equals("Stop Printing")) {
			device.PSclose();
			setVisible(false);
			dispose();
			return true;
		}
		return false;
	}

	public boolean handleEvent(Event e) {
		if (e.id == Event.WINDOW_DESTROY) {
			setVisible(false);
			return (true);
		}
		return super.handleEvent(e);
	}
}

public class Plot2d extends Plot2dCanvas {

	public final static int POINT_SELECT = 3000;
	public final static int BOX_SELECT = 3001;

	boolean ignore_next_up = false;
	String dir = new String("/tmp");

	public Plot2d(int devwidth, int devheight) {
		super(devwidth, devheight);
	}

	Insets getInsets() {
		return new Insets(device.getAscent() + device.getDescent(), 5, 5, 10);
	}

	public void printStart() {
		Component ref = this;
		while (!(ref instanceof Frame))
			ref = ref.getParent();
		FileDialog fd = new FileDialog((Frame) ref, "File Dialog");
		fd.setDirectory(dir);
		fd.setVisible(true);
		if (fd.getFile() == null)
			return;

		try {
			dir = new String(fd.getDirectory());
			PrintStream stream = new PrintStream(new FileOutputStream(dir + "/"
					+ fd.getFile()));
			device.PSopen(stream);
			UtilFrame f = new UtilFrame("Print");
			f.setDevice(device);
			f.setSize(100, 100);
			f.setVisible(true);
		} catch (IOException ex) {
		}
	}

	Point mouse_down = null;
	Rectangle box = null;
	double currentpos[];
	boolean focus = false;

	public boolean handleEvent(Event e) {
		if (ignore_next_up && e.id == Event.MOUSE_UP) {
			ignore_next_up = false;
			return true;
		}

		if (e.shiftDown() && e.id == Event.MOUSE_DOWN) {
			ignore_next_up = true;
			printStart();
		} else if (e.id == Event.MOUSE_EXIT) {
			focus = false;
			repaint();
			return true;
		} else if (e.id == Event.MOUSE_ENTER) {
			focus = true;
			return true;
		} else if (e.id == Event.MOUSE_DOWN) {
			mouse_down = new Point(e.x, devheight - e.y);
			box = new Rectangle(mouse_down);
			return true;
		} else if (e.id == Event.MOUSE_UP) {
			box.add(e.x, devheight - e.y);
			double[] a;
			Power10 d = new Power10();
			String[] s;
			if (box.width != 0 && box.height != 0) {
				s = new String[4];
				double eps;
				a = itransform(box.x, box.y);
				eps = (a[1] - a[0]) / 100.0;
				d.nicestAt(a[0], eps);
				s[0] = d.toString(); // inf_x
				d.nicestAt(a[1], eps);
				s[2] = d.toString(); // inf_y
				a = itransform(box.x + box.width, box.y + box.height);
				eps = (a[1] - a[0]) / 100.0;
				d.nicestAt(a[0], eps);
				s[1] = d.toString(); // sup_x
				d.nicestAt(a[1], eps);
				s[3] = d.toString(); // sup_y
				e.id = BOX_SELECT;
			} else {
				s = new String[2];
				a = itransform(e.x, devheight - e.y);
				d.nicestAt(a[0], res[0]);
				s[0] = d.toString();
				d.nicestAt(a[1], res[1]);
				s[1] = d.toString();
				e.id = POINT_SELECT;
			}
			e.arg = s;
			box = null;
			repaint();
			// Process event in parent container
		} else if (e.id == Event.MOUSE_DRAG) {
			box = new Rectangle(mouse_down);
			box.add(e.x, devheight - e.y);
			repaint();
			return true;
		} else if (e.id == Event.MOUSE_MOVE) {
			currentpos = itransform(e.x, devheight - e.y);
			repaint();
			return true;
		}
		return super.handleEvent(e);
	}

	public void update(Graphics g) {
		paint(g);
	}

	public void paint(Graphics g) {
		if (offscreen != null)
			g.drawImage(offscreen, 0, 0, this);
		if (box != null) {
			g.setColor(Color.red);
			g.drawRect(box.x, devheight - box.y - box.height, box.width,
					box.height);
		}
		if (focus && currentpos != null) {
			g.setColor(Color.red);
			Power10 d = new Power10();
			d.nicestAt(currentpos[0], res[0]);
			String s = "(" + d + ", ";
			d.nicestAt(currentpos[1], res[1]);
			s = s.concat(", " + d);
			g.drawString(s, devwidth - device.stringWidth(s) - 5,
					device.getAscent());
		}
	}
}
