package decatur.de.rigorous.plot2d;

import java.awt.*;

import decatur.de.rigorous.numutil.Power10;

public class Plot2dCanvas extends Canvas {
  boolean debug=false;
  boolean doOrigin = false;
  int width, height;
  int devwidth, devheight;
  public double infx, supx, infy, supy;
  int ShortTickLength;
  int LongTickLength;
  int xLabelWidth;
  int yLabelWidth;
  int xLabelHeight;
  int yLabelHeight;
  double nicex, nicey, dnicex, dnicey;
  double YtickPos[];
  String YtickString[];
  int YtickCount;
  double XtickPos[];
  String XtickString[];
  int XtickCount;
  double sx=1, sy=1, tx=0, ty=0;
  boolean saved = false;
  double tx_save, ty_save;
  Image offscreen=null;
  Polygon path;
  public double[] res = new double[2];
  public final static int AUTOMATIC=0, MANUAL=1, NONE=2;
  int xtickmode = AUTOMATIC;
  int ytickmode = AUTOMATIC;

  PlotDevice device = null;

  Plot2dCanvas (int devwidth, int devheight) {
    this.devwidth = devwidth;
    this.devheight = devheight;
    int l = Math.min(devwidth, devheight);
    ShortTickLength = Math.max(l/150, 2);
    LongTickLength  = Math.max(l/100, 3);
    YtickCount = 0;
    YtickPos = new double[40];
    YtickString = new String[40];
    XtickCount = 0;
    XtickPos = new double[40];
    XtickString = new String[40];
    setSize(devwidth, devheight);
  }

  public Dimension preferredSize() {
    return minimumSize();
  }

  public synchronized Dimension minimumSize() {
    return new Dimension (devwidth, devheight);
  }

  public void addNotify () {
    super.addNotify ();
    device = new PlotDevice (this, devwidth, devheight);
    offscreen = device.getImage();
  }

  double findnice (double a, double b) {
    Power10 d = new Power10();
    d.nicestIn (a, b);
    return d.doubleValue();
  }

  double finddnice (double delta) {
    Power10 dnice = new Power10 ();
    dnice.set (delta,0);
    dnice.m = 1l;
    double d = dnice.doubleValue ();
    if ( d*2.0 > delta ) return d*2.0;
    if ( d*5.0 > delta ) return d*5.0;
    return d*10.0;
  }

  int labelwidth (double inf, double sup, double nice, double dnice) {
    double d = Math.max(Math.abs(inf), Math.abs(sup));
    d = findnice (d,dnice);
    String s="";
    if ( inf<0.0 ) s="-";
    int w1 = device.stringWidth(s+Power10.niceString(d,1e-14));
    int w2 = device.stringWidth(s+Power10.niceString(d+dnice,1e-14));
    return Math.max(w1, w2);
  }

  Insets getInsets() {
    return new Insets(10,5,5,10);
  }

  public void Bounds (double inf_x, double sup_x, double inf_y, double sup_y) {
    infx=inf_x; infy=inf_y; supx=sup_x; supy=sup_y;
    rescale ();
  }  

  void rescale () {
    Insets insets = getInsets();
    Insets in = new Insets(0,0,0,0);
   
    yLabelWidth = xLabelWidth = 0;

    if ( xtickmode==NONE )
      xLabelHeight = 0;
    else xLabelHeight = device.getAscent();
    
    if ( ytickmode==NONE )
      yLabelHeight = 0;
    else yLabelHeight = device.getAscent();

    in.top = yLabelHeight/2 + insets.top;
    in.bottom = (xLabelHeight*3)/2 + insets.bottom;

    height = devheight - in.top - in.bottom;

    sy = height/(supy-infy);
    ty = -sy*infy + in.bottom;

    if ( ytickmode==AUTOMATIC ) {
      nicey  = findnice (infy, supy);
      dnicey = finddnice ((supy-infy)*Math.max(1.0/60, 5.0/devheight));
      Point dp = dtransform (0, dnicey);
      if (dp.y<10) {
	  res = idtransform (0, 10);
	  dnicey = finddnice(res[1]);
	  dp = dtransform (0, dnicey);
      }
      if (debug) System.out.println("nicey "+nicey+" dnicey "+dnicey);
      int ny =2*yLabelHeight/ Math.abs(dp.y);
      //     System.out.println("ny :"+ny);
      ny = (ny/5+1)*5;
      int kinf = (int) ((nicey-infy)/dnicey);
      int ksup = (int) ((supy-nicey)/dnicey);
      YtickCount = 0;
      for (int k=-kinf; k<=ksup; k++) {
	double d = nicey + k*dnicey;
	if ( k%ny==0 ) addYtick(d, Power10.niceString (d,1e-14));
      } 
    }
    if ( ytickmode!=NONE )
      for (int i=0; i<YtickCount; i++)
	yLabelWidth = Math.max(yLabelWidth, device.stringWidth(YtickString[i]));

    in.left =  yLabelWidth + insets.left;
    in.right = insets.right;
    width = devwidth - in.right-in.left;
    sx = width/(supx-infx);
    tx = -sx*infx + in.left;

    if ( xtickmode==AUTOMATIC ) {
      nicex  = findnice (infx, supx);
      dnicex = finddnice ((supx-infx)*Math.max(1.0/60, 5.0/devwidth));
      if (debug) System.out.println("nicex "+nicex+" dnicex "+dnicex);
      xLabelWidth = labelwidth (infx, supx, nicex, dnicex);
    } else if ( xtickmode==MANUAL )
      for (int i=0; i<XtickCount; i++)
	xLabelWidth = Math.max(xLabelWidth, device.stringWidth(XtickString[i]));

    //    System.out.println("xwidth "+xLabelWidth);
    
    if ( xtickmode==AUTOMATIC ) {
      Point p = dtransform (dnicex, 0);
      int nx = xLabelWidth/p.x;
      nx = (nx/10+1)*10;
      int kinf = (int) ((nicex-infx)/dnicex);
      int ksup = (int) ((supx-nicex)/dnicex);
      XtickCount = 0;
      for (int k=-kinf; k<=ksup; k++) {
	double d = nicex + k*dnicex;
	if ( k%nx==0 ) 
	  addXtick(d, Power10.niceString (d, 1e-14));
      } 
    }
    
    res = idtransform(1, 1);
  }

  public void erase () {
    device.close ();
    device.open ();
  }

  public void setDoOrigin (boolean f) {
    doOrigin = true;
  }
 
  public void setYmode (int mode) {
    ytickmode=mode;
    YtickCount = 0;
  } 

  public void setXmode (int mode) {
    xtickmode=mode;
    XtickCount = 0;
  } 

  public void addYtick(double pos, String s) {
    YtickPos[YtickCount] = pos;
    YtickString[YtickCount] = new String(s);
    YtickCount ++;
  }

  public void addXtick(double pos, String s) {
    XtickPos[XtickCount] = pos;
    XtickString[XtickCount] = new String(s);
    XtickCount ++;
  }

  public void drawTick (double d, String s, char mode) {
    int l = ShortTickLength;

    if ( s!=null ) l = LongTickLength;
    if ( mode=='x' ) {
      Point p = transform (d, infy);
      if ( s!=null )
	  device.drawString (s, p.x, p.y-4*xLabelHeight/3, PlotDevice.CENTER);  
      device.drawLine (p.x, p.y, p.x, p.y+l);
    } else {
      Point p = transform (infx, d);
      if ( s!=null )
        device.drawString (s, p.x-3, p.y-yLabelHeight/2, PlotDevice.RIGHT);  
      device.drawLine (p.x, p.y, p.x+l, p.y);
    }
    
  }
  
  public void drawAxes () {
    device.setColor (Color.black);
    device.drawRect (1, 1, devwidth-2, devheight-1);
    //    device.drawRect (0, 0, devwidth-2, devheight-2);
    if ( doOrigin ) {
	if ( infy<0.0 && supy>0.0 )
	    drawLine (infx, 0.0, supx, 0.0);
	if ( infx<0.0 && supx>0.0 )
	    drawLine (0.0, infy, 0.0, supy);
    }

    drawRect (infx, infy, supx-infx, supy-infy, false);

    if ( xtickmode==AUTOMATIC )
      drawTicksAutomatic (infx, supx, nicex, dnicex, 'x');
    if ( xtickmode!=NONE )
      for (int i=0; i<XtickCount; i++)
	drawTick (XtickPos[i], XtickString[i], 'x');
    
    if ( ytickmode==AUTOMATIC )
      drawTicksAutomatic (infy, supy, nicey, dnicey, 'y');
    if ( ytickmode!=NONE )
      for (int i=0; i<YtickCount; i++)
	drawTick (YtickPos[i], YtickString[i], 'y');
  }

  void drawTicksAutomatic (double inf, double sup, double nice,
			   double dnice, char mode) {
    int kinf = (int) ((nice-inf)/dnice);
    int ksup = (int) ((sup-nice)/dnice);

    for (int k=-kinf; k<=ksup; k++) {
      double d = nice + k*dnice;
      drawTick (d, null, mode);
    }
  }

  public void save () {
    if ( saved ) restore ();
    else saved = true;
    tx_save = tx;
    ty_save = ty;
  }

  public void restore () {
    saved = false;
    tx = tx_save;
    ty = ty_save;
  }

  public Point transform (double x, double y) {
    return new Point((int) Math.round(sx*x+tx), (int) Math.round(sy*y+ty));
  }

  public Point dtransform (double dx, double dy) {
    return new Point((int) Math.round(sx*dx), (int) Math.round(sy*dy));
  }

  public double[] itransform (int x, int y) {
    double p[] = new double[2];
    p[0] = (x-tx)/sx;
    p[1] = (y-ty)/sy;
    return p;
  }

  public double[] idtransform (int x, int y) {
    double p[] = new double[2];
    p[0] = x/sx;
    p[1] = y/sy;
    return p;
  }

  public void translate(double dx, double dy) {
    tx += dx*sx;
    ty += dy*sy;
  }

  public void setColor (Color color) {
    device.setColor (color);
  }

  public void drawPoint(double x, double y) {
    Point p = transform (x, y);
    device.drawRect (p.x, p.y, 1, 1);
  }

  public void drawLine(double x, double y, double x2, double y2) {
    Point p = transform (x, y);
    Point p2 = transform (x2, y2);
    device.drawLine (p.x, p.y, p2.x, p2.y);
  }

  public void drawRect(double x, double y, double w, double h, boolean fill) {
    Point p = transform (x,y);
    Point dp = dtransform (w, h);
    if ( fill )
      device.fillRect (p.x, p.y, dp.x, dp.y);
    else
      device.drawRect (p.x, p.y, dp.x, dp.y);
  }

  public void drawOval (double x, double y, double w, double h,
			boolean fill) {
    Point p = transform (x,y);
    Point dp = dtransform (w, h);
    device.drawOval (p.x, p.y, dp.x, dp.y, fill);
  }

  public void newpath (double x, double y) {
    path = new Polygon();
    lineto (x, y);
  }

  public void lineto (double x, double y) {
    Point p = transform (x,y);
    path.addPoint(p.x, p.y);
  }

  /*  public void expandpath (double x, double y) {
    Point p = transform (x,y);
    for (int i=0; i<path.npoints; i++) {
      if ( xpoints[i]>x ) xpoints[i]++;
      else xpoints[i]--;
      if ( ypoints[i]>y ) ypoints[i]++;
      else ypoints[i]--;
    }
  }*/

  public void fillpath () {
    device.fillPath (path);
  }

  void graph() {};

  public void update (Graphics g) {
    paint (g);
  }

  public void paint(Graphics g) {
    if ( offscreen!=null )
      g.drawImage (offscreen, 0, 0, this);
  }
}

